@@ -1,5 +1,8 @@
 #' SObject Basic Information
 #' 
+#' @description
+#' \lifecycle{maturing}
+#' 
 #' Describes the individual metadata for the specified object.
 #' 
 #' @importFrom methods as
@@ -12,11 +15,14 @@
 #' @param ... arguments passed to \code{\link{sf_control}}
 #' @template verbose
 #' @return \code{list}
+#' @seealso \href{https://developer.salesforce.com/docs/atlas.en-us.api_rest.meta/api_rest/resources_sobject_describe.htm}{REST API Documentation}, \href{https://developer.salesforce.com/docs/atlas.en-us.api_rest.meta/api_rest/dome_sobject_describe.htm}{REST API Example}
 #' @examples
 #' \dontrun{
 #' account_metadata <- sf_describe_objects("Account")
 #' account_metadata_SOAP <- sf_describe_objects("Account", api_type="SOAP")
 #' multiple_objs_metadata <- sf_describe_objects(c("Contact", "Lead"))
+#' 
+#' account_metadata_REST <- sf_describe_objects("Account", api_type="REST")
 #' }
 #' @export
 sf_describe_objects <- function(object_names,
@@ -29,20 +35,22 @@ sf_describe_objects <- function(object_names,
   
   listed_objects <- sf_list_objects()
   valid_object_names <- sapply(listed_objects$sobjects, FUN=function(x){x$name})
-
   not_matched_objs <- setdiff(object_names$sObjectType, valid_object_names)
   if(length(not_matched_objs) > 0){
-    message(sprintf("Skipping the following object(s) for not matching the name of an existing object: %s", paste0(not_matched_objs, collapse=", ")))
+    message_w_errors_listed(main_text = paste0("Skipping the following object(s) ", 
+                                               "for not matching the name of an ", 
+                                               "existing object:"), 
+                            not_matched_objs)
   }
 
-  object_names <- object_names %>%
-    filter(sObjectType %in% valid_object_names) %>% 
-    as.data.frame()
+  object_names <- as.data.frame(
+    object_names[object_names$sObjectType %in% valid_object_names, , drop=FALSE]
+  )
   
   if(which_api == "REST"){
     resultset <- list()
     for(i in 1:nrow(object_names)){
-      describe_object_url <- make_rest_objects_url(object_names[i,"sObjectType"])
+      describe_object_url <- make_rest_describe_url(object_names[i, "sObjectType"])
       httr_response <- rGET(url = describe_object_url)
       if(verbose){
         make_verbose_httr_message(httr_response$request$method,
@@ -51,14 +59,17 @@ sf_describe_objects <- function(object_names,
       }
       catch_errors(httr_response)
       response_parsed <- content(httr_response, as="parsed", encoding="UTF-8")
-      # TODO: Need to fix!!!!
-      resultset[[i]] <- response_parsed$objectDescribe
+      resultset[[i]] <- response_parsed
     }
   } else if(which_api == "SOAP"){
+    api_type <- match.arg(api_type)
+    # determine how to pass along the control args 
+    all_args <- list(...)
     control_args <- return_matching_controls(control)
     control_args$api_type <- "SOAP"
     control_args$operation <- "describeSObjects"
-    control <- do.call("sf_control", control_args)
+    control <- filter_valid_controls(control)
+    
     r <- make_soap_xml_skeleton(soap_headers = control)
     xml_dat <- build_soap_xml_from_list(input_data = object_names$sObjectType,
                                         operation = "describeSObjects",
@@ -76,7 +87,7 @@ sf_describe_objects <- function(object_names,
                                 request_body)
     }
     catch_errors(httr_response)
-    response_parsed <- content(httr_response, encoding="UTF-8")
+    response_parsed <- content(httr_response, as="parsed", encoding="UTF-8")
     
     invisible(capture.output(
       resultset <- response_parsed %>%
@@ -89,7 +100,7 @@ sf_describe_objects <- function(object_names,
         })
     ))
   } else {
-    stop("Unsupported API type. Set api_type equal to 'REST' or 'SOAP'.")
+    catch_unknown_api(api_type, supported=c("REST", "SOAP"))
   }
   return(resultset)
 }
\ No newline at end of file
