@@ -1,8 +1,5 @@
 context("REST API")
 
-salesforcer_token <- readRDS("salesforcer_token.rds")
-sf_auth(token = salesforcer_token)
-
 test_that("testing REST API Functionality", {
   
   n <- 2
@@ -14,16 +11,17 @@ test_that("testing REST API Functionality", {
   # sf_create ------------------------------------------------------------------  
   created_records <- sf_create(new_contacts, object_name = object, api_type="REST")
   expect_is(created_records, "tbl_df")
-  expect_equal(names(created_records), c("id", "success", "errors"))
+  expect_equal(names(created_records), c("id", "success"))
   expect_equal(nrow(created_records), n)
+  expect_is(created_records$success, "logical")
   
   # sf_retrieve ----------------------------------------------------------------  
-  retrieved_records <- sf_retrieve(ids=created_records$id, 
-                                   fields=c("FirstName", "LastName"), 
+  retrieved_records <- sf_retrieve(ids = created_records$id, 
+                                   fields = c("FirstName", "LastName"), 
                                    object_name = object, 
-                                   api_type="REST")
+                                   api_type = "REST")
   expect_is(retrieved_records, "tbl_df")
-  expect_equal(names(retrieved_records), c("Id", "FirstName", "LastName"))
+  expect_equal(names(retrieved_records), c("sObject", "Id", "FirstName", "LastName"))
   expect_equal(nrow(retrieved_records), n)
   
   # FYI: Will not find newly created records because records need to be indexed
@@ -34,7 +32,7 @@ test_that("testing REST API Functionality", {
   # sf_search ------------------------------------------------------------------
   searched_records <- sf_search(my_sosl, is_sosl=TRUE, api_type="REST") 
   expect_is(searched_records, "tbl_df")
-  expect_equal(names(searched_records), c("sObject", "Id", "FirstName", "LastName", "My_External_Id__c"))
+  expect_named(searched_records, c("sObject", "Id", "FirstName", "LastName"))
   expect_equal(nrow(searched_records), 3)
   
   my_soql <- sprintf("SELECT Id, 
@@ -56,8 +54,9 @@ test_that("testing REST API Functionality", {
   # sf_update ------------------------------------------------------------------
   updated_records <- sf_update(queried_records, object_name = object, api_type="REST")
   expect_is(updated_records, "tbl_df")
-  expect_equal(names(updated_records), c("id", "success", "errors"))
+  expect_equal(names(updated_records), c("id", "success"))
   expect_equal(nrow(updated_records), n)
+  expect_is(updated_records$success, "logical")
   
   new_record <- tibble(FirstName = "Test",
                        LastName = paste0("REST-Contact-Upsert-", n+1), 
@@ -70,9 +69,10 @@ test_that("testing REST API Functionality", {
                                 external_id_fieldname = "My_External_Id__c", 
                                 api_type = "REST")
   expect_is(upserted_records, "tbl_df")
-  expect_equal(names(upserted_records), c("id", "success", "errors", "created"))
+  expect_equal(names(upserted_records), c("id", "success", "created"))
   expect_equal(nrow(upserted_records), nrow(upserted_records))
-  
+  expect_equal(upserted_records$success, c(TRUE, TRUE, TRUE))
+  expect_equal(upserted_records$created, c(FALSE, FALSE, TRUE))
   
   # sf_create_attachment -------------------------------------------------------
   attachment_details <- tibble(Name = c("salesforcer Logo"),
@@ -94,6 +94,8 @@ test_that("testing REST API Functionality", {
   ids_to_delete <- unique(c(upserted_records$id[!is.na(upserted_records$id)], queried_records$Id))
   deleted_records <- sf_delete(ids_to_delete, object_name = object, api_type = "REST")
   expect_is(deleted_records, "tbl_df")
-  expect_equal(names(deleted_records), c("id", "success", "errors"))
+  expect_equal(names(deleted_records), c("id", "success"))
   expect_equal(nrow(deleted_records), length(ids_to_delete))
+  expect_is(deleted_records$success, "logical")
+  expect_true(all(deleted_records$success))
 })
